﻿using Android.App;
using Android.OS;
using Android.Support.V7.App;
using Android.Runtime;
using Android.Widget;
using Android.Views;
using System.Collections.Generic;
using System.Threading;
using System;
using static Android.App.ActivityManager;
using Android.Content;
using Android.Content.PM;
using Android.Hardware;

namespace StanUrzadzenia
{
    [Activity(Label = "@string/app_name", Theme = "@style/AppTheme", MainLauncher = true)]
    public class MainActivity : AppCompatActivity
    {
        ListView listView;

        class _Handler : Handler
        {
            private MainActivity activity;

            public _Handler(MainActivity activity)
            {
                this.activity = activity;
            }

            public override void HandleMessage(Message message)
            {
                activity.odswiezListe();
            }
        };

        _Handler handler;        

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);
            // Set our view from the "main" layout resource
            SetContentView(Resource.Layout.activity_main);

            TextView naglowek = FindViewById<TextView>(Resource.Id.naglowek);
            naglowek.Text += "\n" + Build.Manufacturer + " " + Build.Model + " (" + Build.Product + ")";

            listView = FindViewById<ListView>(Resource.Id.listView);
            odswiezListe();

            const int coIleMilisekundOswiezacListe = 2000; //2s
            handler = new _Handler(this);
            ThreadStart ts = new ThreadStart( //wątek .NET, alternatywnie Java.Lang.Thread
                () =>
                {
                    while (true)
                    {
                        //w osobnym watku
                        handler.SendEmptyMessage(0);
                        try
                        {
                            Thread.Sleep(coIleMilisekundOswiezacListe);
                        }
                        catch //(ThreadInterruptedException exc)
                        {
                        }
                    }
                });
            Thread timer = new Thread(ts);
            timer.Start();
        }

        class BatteryReceiver : BroadcastReceiver
        {
            public override void OnReceive(Context context, Intent intent)
            {
                int bateriaPoziom = intent.GetIntExtra(BatteryManager.ExtraLevel, -1);
                if (bateriaPoziom < 10) Toast.MakeText(context, "Nisko poziom baterii", ToastLength.Short).Show();
            }
        }

        BatteryReceiver batteryReceiver;

        private void odswiezListe()
        {
            int pozycja = listView.FirstVisiblePosition;
            View v = listView.GetChildAt(0);
            int top = (v == null) ? 0 : v.Top;

            List<UrządzenieInfo> listaUrzadzeń = new List<UrządzenieInfo>();
            /*
            listaUrzadzeń.Add(odczytajWersjęSystemu(this));
            listaUrzadzeń.Add(odczytajStanPamięci(this));
            listaUrzadzeń.Add(odczytajStanGłównejKartyPamięci(this));
            listaUrzadzeń.Add(odczytajStanDodatkowejKartyPamięci(this));
            listaUrzadzeń.Add(odczytajStanProcesora(this));
            */

            //tu dodawać opisy stanu kolejnych urzadzen
            batteryReceiver = new BatteryReceiver();
            IntentFilter intentFilter = new IntentFilter(Intent.ActionBatteryChanged);
            Intent intent = this.ApplicationContext.RegisterReceiver(batteryReceiver, intentFilter);
            listaUrzadzeń.Add(odczytStanuBaterii(this, intent));

            listaUrzadzeń.Add(odczytajObecnośćDodatkowychSensorów(this));
            listaUrzadzeń.Add(odczytajObecnośćCzcujników(this));

            UrzadzeniaAdapter a = new UrzadzeniaAdapter(this, Resource.Layout.wiersz, listaUrzadzeń);
            listView.Adapter = a;

            listView.SetSelectionFromTop(pozycja, top);
        }

        protected override void OnResume()
        {
            base.OnResume();
            //Toast.MakeText(this, "Rozpoczynam odczyty czujników", ToastLength.Long).Show();
        }

        protected override void OnPause()
        {
            if (batteryReceiver != null)
            {
                Toast.MakeText(this, "Zatrzymuję!", ToastLength.Long).Show();
                this.ApplicationContext.UnregisterReceiver(batteryReceiver);
            }
            base.OnPause();            
        }

        #region Metody odczytujące stan urządzenia
        static private UrządzenieInfo odczytajWersjęSystemu(Activity activity)
        {
            UrządzenieInfo systemInfo = new UrządzenieInfo();
            systemInfo.nazwa = "Wersja systemu";
            systemInfo.typInformacji = TypInformacji.Inne;
            systemInfo.polozeniePaska = -1;
            systemInfo.opis = "\nBieżąca data i czas: " + DateTime.Now.ToString();
            systemInfo.opis += "\nBieżąca data: " + DateTime.Now.ToLongTimeString();
            systemInfo.opis += "\nBieżący czas: " + DateTime.Now.ToLongDateString();
            systemInfo.opis += "\nProducent: " + Build.Manufacturer;
            systemInfo.opis += "\nUrządzenia: " + Build.Device;
            systemInfo.opis += "\nS/N: " + Build.Serial;
            systemInfo.opis += "\nWersja: " + Build.VERSION.Release + " (" + Build.VERSION.Incremental + ")";
            systemInfo.opis += "\nPoziom API: " + Build.VERSION.SdkInt;
            //systemInfo.opis += "\nTyp CPU: " + Build.CpuAbi; //deprecated
            systemInfo.opis += "\nTypy CPU: ";
            foreach(string abi in Build.SupportedAbis) //API 21
            {
                systemInfo.opis += "\n\t" + abi;
            }
            systemInfo.opis += "\nWyświetlacz: " + Build.Display;
            return systemInfo;
        }

        static private UrządzenieInfo odczytajStanPamięci(Activity activity)
        {
            //Pamięć wewnętrzna
            UrządzenieInfo pi = new UrządzenieInfo();
            pi.typInformacji = TypInformacji.StanUrzadzenia;
            pi.nazwa = "Pamięć (RAM)";

            ActivityManager am = (ActivityManager)activity.GetSystemService(ActivityService);
            MemoryInfo mi = new MemoryInfo();
            am.GetMemoryInfo(mi);
            long pamięćDostępna = mi.AvailMem / 1024; //kB
                                                      //mi.totalMem
            long pamięćCałkowita = LinuxHelper.ReadRAMTotalSizeKB();
            long pamięćZajęta = pamięćCałkowita - pamięćDostępna;
            pi.opis = "Dostępna pamięć: " + pamięćDostępna + "/" + pamięćCałkowita;
            pi.polozeniePaska = (int)(pamięćZajęta * 100 / pamięćCałkowita);
            return pi;
        }

        static private Tuple<string, int> pobierzStanPamięci(Java.IO.File memoryDirectory)
        {
            StatFs systemPlików = new StatFs(memoryDirectory.Path);
            long wolneMiejsce =
                    systemPlików.AvailableBlocksLong *
                            systemPlików.BlockSizeLong
                            / 1024 / 1024; //MB
            long rozmiar =
                    systemPlików.BlockCountLong *
                            systemPlików.BlockSizeLong
                            / 1024 / 1024; //MB
            long zajęteMiejsce = rozmiar - wolneMiejsce;

            String opis = "Ścieżka: " + memoryDirectory.Path + "\n";
            opis += "Wolne miejsce: " + wolneMiejsce + " / " + rozmiar;
            int procent = (int)(zajęteMiejsce * 100 / rozmiar);

            Tuple<string, int> tuple = new Tuple<string, int>(opis, procent);
            return tuple;
        }

        static private UrządzenieInfo odczytajStanGłównejKartyPamięci(Activity activity)
        {
            Tuple<string, int> tuple = pobierzStanPamięci(Android.OS.Environment.RootDirectory);

            UrządzenieInfo kartaPamięciInfo = new UrządzenieInfo();
            kartaPamięciInfo.typInformacji = TypInformacji.StanUrzadzenia;
            kartaPamięciInfo.nazwa = "Głowna karta pamięci";
            kartaPamięciInfo.opis = tuple.Item1;
            kartaPamięciInfo.polozeniePaska = tuple.Item2;
            return kartaPamięciInfo;
        }

        static private UrządzenieInfo odczytajStanDodatkowejKartyPamięci(Activity activity)
        {
            UrządzenieInfo kartaPamięciInfo = new UrządzenieInfo();
            kartaPamięciInfo.typInformacji = TypInformacji.StanUrzadzenia;
            kartaPamięciInfo.nazwa = "Dodatkowa karta pamięci";

            if (Android.OS.Environment.ExternalStorageState.Equals(Android.OS.Environment.MediaMounted))
            {
                Tuple<string, int> tuple = pobierzStanPamięci(Android.OS.Environment.ExternalStorageDirectory);
                kartaPamięciInfo.opis = tuple.Item1;
                kartaPamięciInfo.polozeniePaska = tuple.Item2;
            }
            else
            {
                kartaPamięciInfo.opis = "Brak";
                kartaPamięciInfo.polozeniePaska = -1;
            }
            return kartaPamięciInfo;
        }

        static private UrządzenieInfo odczytajStanProcesora(Activity activity)
        {
            UrządzenieInfo procesorInfo = new UrządzenieInfo();
            procesorInfo.typInformacji = TypInformacji.StanUrzadzenia;
            procesorInfo.nazwa = "Procesor (CPU)";
            int procesorObciążenie = (int)Math.Round(100 * LinuxHelper.ReadCPUUsage());
            String opisProcesora = LinuxHelper.ReadCPUInfo();
            procesorInfo.opis = opisProcesora + "\n";
            procesorInfo.opis += "Obciążenie procesora: " + procesorObciążenie + "%";
            procesorInfo.polozeniePaska = procesorObciążenie;
            return procesorInfo;
        }
        #endregion

        static private UrządzenieInfo odczytStanuBaterii(Activity activity, Intent intent)
        {
            UrządzenieInfo bateriaInfo = new UrządzenieInfo();
            bateriaInfo.typInformacji = TypInformacji.StanUrzadzenia;
            bateriaInfo.nazwa = "Bateria";
            if(intent.GetBooleanExtra(BatteryManager.ExtraPresent, false))
            {
                string bateriaTechnologia = intent.GetStringExtra(BatteryManager.ExtraTechnology);
                BatteryStatus bateriaStan = (BatteryStatus)intent.GetIntExtra(BatteryManager.ExtraStatus, -1);
                string bateriaStanOpis = bateriaStan.ToString();
                BatteryPlugged batteryPlugged = (BatteryPlugged)intent.GetIntExtra(BatteryManager.ExtraPlugged, -1);
                string bateriaPodłączonaOpis = batteryPlugged.ToString();
                int bateriaTemperatura = intent.GetIntExtra(BatteryManager.ExtraTemperature, -1);
                int bateriaNapięcie = intent.GetIntExtra(BatteryManager.ExtraVoltage, -1);
                bateriaInfo.opis = "Technologia: " + bateriaTechnologia;
                bateriaInfo.opis += "\nStan: " + bateriaStanOpis;
                bateriaInfo.opis += "\nPodłączona: " + bateriaPodłączonaOpis;
                bateriaInfo.opis += "\nTemparatura: " + bateriaTemperatura;
                bateriaInfo.opis += "\nNapięcie: " + bateriaNapięcie;

                int bateriaPoziom = intent.GetIntExtra(BatteryManager.ExtraLevel, -1);
                int bateriaSkala = intent.GetIntExtra(BatteryManager.ExtraScale, -1);
                bateriaInfo.opis += "\nPoziom: " + bateriaPoziom + "/" + bateriaSkala;
                bateriaInfo.polozeniePaska = bateriaPoziom * 100 / bateriaSkala;
            }   
            else
            {
                bateriaInfo.polozeniePaska = -1;
                bateriaInfo.opis = "Brak baterii";
            }
            return bateriaInfo;
        }

        private static string taknie(bool b) { return b ? "tak" : "nie"; }

        static private UrządzenieInfo odczytajObecnośćDodatkowychSensorów(Activity activity)
        {
            //Dodatkowe urządzenia
            UrządzenieInfo dodatkoweUrządzeniaInfo = new UrządzenieInfo();
            dodatkoweUrządzeniaInfo.typInformacji = TypInformacji.Inne;
            dodatkoweUrządzeniaInfo.nazwa = "Dodatkowe urządzenia";
            dodatkoweUrządzeniaInfo.polozeniePaska = -1;

            PackageManager pm = activity.PackageManager;
            bool czyTelefonObecny = pm.HasSystemFeature(PackageManager.FeatureTelephony);
            dodatkoweUrządzeniaInfo.opis = "Telefon: " + taknie(czyTelefonObecny);
            if(czyTelefonObecny)
            {
                dodatkoweUrządzeniaInfo.opis += pm.HasSystemFeature(PackageManager.FeatureTelephonyGsm) ? " (GSM)" : " (inny)";
            }
            bool czyAparatFotograficznyObecny = pm.HasSystemFeature(PackageManager.FeatureCamera);
            dodatkoweUrządzeniaInfo.opis += "\nAparat: " + taknie(czyAparatFotograficznyObecny);
            if(czyAparatFotograficznyObecny)
            {
                dodatkoweUrządzeniaInfo.opis += "\n\tAutomatyczna regulacja ostrości: " + taknie(pm.HasSystemFeature(PackageManager.FeatureCameraAutofocus));
                dodatkoweUrządzeniaInfo.opis += "\n\tLampa błyskowa: " + taknie(pm.HasSystemFeature(PackageManager.FeatureCameraFlash));
            }

            dodatkoweUrządzeniaInfo.opis += "\nCzujnik światła: " + taknie(pm.HasSystemFeature(PackageManager.FeatureSensorLight));
            dodatkoweUrządzeniaInfo.opis += "\nCzujnik zbliżeniowy: " + taknie(pm.HasSystemFeature(PackageManager.FeatureSensorProximity));
            dodatkoweUrządzeniaInfo.opis += "\nEkran wielodotykowy: " + taknie(pm.HasSystemFeature(PackageManager.FeatureTouchscreenMultitouch));

            return dodatkoweUrządzeniaInfo;
        }

        static private string opisCzujnika(IList<Sensor> lista, string nazwa)
        {
            string opis = nazwa + ": ";
            if (lista.Count > 1) opis += "\n";
            if(lista.Count > 0)
            {
                for(int i = 0; i < lista.Count; ++i)
                {
                    if (lista.Count > 1) opis += "\t";
                    Sensor czujnik = lista[i];
                    opis += czujnik.Name + ", wersja " + czujnik.Version + ", dostawca " + czujnik.Vendor;                    
                }
            }
            return opis;
        }

        static private UrządzenieInfo odczytajObecnośćCzcujników(Activity activity)
        {
            UrządzenieInfo czujnikiInfo = new UrządzenieInfo();
            czujnikiInfo.typInformacji = TypInformacji.Inne;
            czujnikiInfo.polozeniePaska = -1;
            czujnikiInfo.nazwa = "Czujniki";

            SensorManager sm = (SensorManager)activity.GetSystemService(Context.SensorService);
            czujnikiInfo.opis += opisCzujnika(sm.GetSensorList(SensorType.Accelerometer), "akcelerometr");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.Gyroscope), "żyroskop");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.Light), "światło");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.MagneticField), "pole magnetyczne");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.Orientation), "orientacja");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.Pressure), "ciśnienie");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.Temperature), "temperatura");
            czujnikiInfo.opis += "\n" + opisCzujnika(sm.GetSensorList(SensorType.AmbientTemperature), "temperatura otoczenia");

            return czujnikiInfo;
        }
    }
}